#! /usr/bin/env python

import sys

def usage(msg):
   pname = sys.argv[0]
   sys.stderr.write("%s: %s\n" % (pname, msg))
   sys.stderr.write("Usage: %s [-[12][-+]] [-a artist] [-l album] [-g genre] [-n track] [-N total_tracks] [-t title] [-y year] filename\n" % pname)
   sys.exit(1)

from getopt import getopt, GetoptError
try:
   optlist, args = getopt(sys.argv[1:], "1:2:a:g:l:n:N:t:y:",
      ["id3v1=", "id3v2=", "artist=", "album=", "genre=", "track=", "tracks=",
         "title=", "year="]
   )
except GetoptError:
   usage("invalid option")

write_id3v1 = True
write_id3v2 = True

artist = None
album = None
genre = None
track = None
tracks = None
title = None
year = None

for _opt, _arg in optlist:
   if _opt == "-1":
      if _arg == '-':
         write_id3v1 = False
   elif _opt == "--id3v1":
      if _arg.lower() in ("no", "off"):
         write_id3v1 = False
   elif _opt == "-2":
      if _arg == "-":
         write_id3v2 = False
   elif _opt == "--id3v2":
      if _arg.lower() in ("no", "off"):
         write_id3v2 = False
   elif _opt in ("-a", "--artist"):
      artist = _arg
   elif _opt in ("-l", "--album"):
      album = _arg
   elif _opt in ("-g", "--genre"):
      genre = _arg
   elif _opt in ("-n", "--track"):
      try:
         track = int(_arg)
      except ValueError:
         track = None
   elif _opt in ("-N", "--tracks"):
      try:
         tracks = int(_arg)
      except ValueError:
         tracks = None
   elif _opt in ("-t", "--title"):
      title = _arg
   elif _opt in ("-y", "--year"):
      year = _arg
try:
   del _opt, _arg
except NameError:
   pass

if args:
   if len(args) > 1:
      usage("too many arguments")

   filename = args[0]

else:
   usage("too few arguments")


from os import utime
from os.path import getmtime
mtime = getmtime(filename)

import eyeD3

versions = []
if write_id3v2:
   versions.append(eyeD3.ID3_V2)
if write_id3v1:
   versions.append(eyeD3.ID3_V1)

for version in versions:
   id3tag = eyeD3.Tag()
   id3tag.link(filename, version)
   id3tag.setVersion(version)

   if album: id3tag.setAlbum(unicode(album, "latin1"))
   if artist: id3tag.setArtist(unicode(artist, "latin1"))
   if track or tracks: id3tag.setTrackNum((track, tracks))
   if title: id3tag.setTitle(unicode(title, "latin1"))
   if genre: id3tag.setGenre(genre)
   if year: id3tag.setDate(year)

   id3tag.update()
utime(filename, (mtime, mtime))
